using System;

namespace Smdn.Protocols.Imap4 {
  // 7.4.2. FETCH Response
  //       ENVELOPE
  //         An address structure is a parenthesized list that describes an
  //         electronic mail address.  The fields of an address structure
  //         are in the following order: personal name, [SMTP]
  //         at-domain-list (source route), mailbox name, and host name.
  public class ImapAddress {
    public string Name {
      get { return name; }
    }

    public string AtDomailList {
      get { return atDomainList; }
    }

    public string Mailbox {
      get { return mailbox; }
    }

    public string Host {
      get { return host; }
    }

    public ImapAddress(string name, string atDomainList, string mailbox, string host)
    {
      this.name = name;
      this.atDomainList = atDomainList;
      this.mailbox = mailbox;
      this.host = host;
    }

    public override string ToString()
    {
      return string.Format("{{Name={0}, AtDomainList={1}, Mailbox={2}, Host={3}}}", name, atDomainList, mailbox, host);
    }

    private readonly string name;
    private readonly string atDomainList;
    private readonly string mailbox;
    private readonly string host;
  }
}

/*
address         = "(" addr-name SP addr-adl SP addr-mailbox SP
                  addr-host ")"
addr-adl        = nstring
                    ; Holds route from [RFC-2822] route-addr if
                    ; non-NIL
addr-host       = nstring
                    ; NIL indicates [RFC-2822] group syntax.
                    ; Otherwise, holds [RFC-2822] domain name
addr-mailbox    = nstring
                    ; NIL indicates end of [RFC-2822] group; if
                    ; non-NIL and addr-host is NIL, holds
                    ; [RFC-2822] group name.
                    ; Otherwise, holds [RFC-2822] local-part
                    ; after removing [RFC-2822] quoting
addr-name       = nstring
                    ; If non-NIL, holds phrase from [RFC-2822]
                    ; mailbox after removing [RFC-2822] quoting
 */