using System;

namespace Smdn.Protocols.Imap4 {
  public class ImapCapability {
    public ImapKnownCapabilities KnownCapabilities {
      get { return knownCapabilities; }
      internal set { knownCapabilities = value; }
    }

    public string[] AllCapabilities {
      get { return allCapabilities; }
      internal set { allCapabilities = value; }
    }

    internal ImapCapability()
      : this(ImapKnownCapabilities.None)
    {
    }

    internal ImapCapability(ImapKnownCapabilities knownCapabilities)
      : this(knownCapabilities, new string[] {})
    {
    }

    internal ImapCapability(ImapKnownCapabilities knownCapabilities, string[] allCapabilities)
    {
      this.knownCapabilities = knownCapabilities;
      this.allCapabilities = allCapabilities;
    }

    public bool Has(ImapKnownCapabilities capability)
    {
      return (0 != (knownCapabilities & capability));
    }

    public bool Has(string capability)
    {
      foreach (var capa in allCapabilities) {
        if (capability.Equals(capa, StringComparison.InvariantCultureIgnoreCase))
          return true;
      }

      return false;
    }

    public override string ToString()
    {
      return string.Format("{{KnownCapabilities={0}, AllCapabilities={1}}}",
                           knownCapabilities,
                           string.Join(" ", allCapabilities));
    }

    private ImapKnownCapabilities knownCapabilities;
    private string[] allCapabilities;
  }
}
