using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Protocols.Imap4.Protocol {
  // Layers:
  //   Client.ImapClient
  //   Client.ImapClientAgent
  //   Session.ImapSessionAgent
  //   Session.ImapSession
  //   Transaction.ImapTransactionAgent
  //   Transaction.ImapTransaction/Transaction.ImapRequest/Transaction.ImapResponse
  // * Protocol.ImapConnection/Protocol.ImapProtocolCommand/Protocol.ImapProtocolResponse
  //   (IMAP4)

  // handles '6. Client Commands'
  public class ImapProtocolCommand {
    public string Tag {
      get { return tag; }
    }

    public string Command {
      get { return command; }
    }

    public string[] Parameters {
      get { return parameters; }
    }

    public ImapProtocolCommand(string command)
      : this(command, string.Empty, new string[] {})
    {
    }

    public ImapProtocolCommand(string command, string tag, string[] parameters)
    {
      this.command = command;
      this.tag = tag;
      this.parameters = parameters;
    }

    public override string ToString()
    {
      return string.Format("{{Command={0}, Tag={1}}}", command, tag);
    }

    public byte[] Format()
    {
      var commandString = new StringBuilder();

      if (command != null)
        commandString.AppendFormat("{0} {1}", tag, command);

      for (int i = 0; i < parameters.Length; i++) {
        if (!(command == null && i == 0))
          commandString.Append(" ");
        commandString.Append(parameters[i]);
      }

      commandString.Append("\r\n");

      return Encoding.ASCII.GetBytes(commandString.ToString());
    }

    private readonly string command;
    private readonly string tag;
    private readonly string[] parameters;
  }
}