using System;
using System.Collections.Generic;

namespace Smdn.Protocols.Imap4.Transaction {
  // Layers:
  //   Client.ImapClient
  //   Client.ImapClientAgent
  //   Session.ImapSessionAgent
  //   Session.ImapSession
  //   Transaction.ImapTransactionAgent
  // * Transaction.ImapTransaction/Transaction.ImapRequest/Transaction.ImapResponse
  //   Protocol.ImapConnection/Protocol.ImapProtocolCommand/Protocol.ImapProtocolResponse
  //   (IMAP4)

  public class ImapRequest {
    public ImapTransactionMethod Method {
      get { return method; }
    }

    public Dictionary<string, string> Parameters {
      get { return parameters; }
    }

    public int Timeout {
      get { return timeout; }
    }

    internal DateTime RequestedTime {
      get; set;
    }

    internal ImapRequest(ImapTransactionMethod method, int timeout)
      : this(method, timeout, null)
    {
    }

    internal ImapRequest(ImapTransactionMethod method, int timeout, IDictionary<string, string> parameters)
    {
      this.method = method;
      this.timeout = timeout;

      if (parameters != null) {
        foreach (var pair in parameters) {
          this.parameters.Add(pair.Key, pair.Value);
        }
      }
    }

    public override string ToString()
    {
      return string.Format("{{Method={0}, Parameters={1}}}", method, parameters.Count);
    }

    private readonly Dictionary<string, string> parameters = new Dictionary<string, string>();
    private readonly ImapTransactionMethod method;
    private readonly int timeout;
  }
}
