using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class FetchTransaction : TransactionBase {

    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessFetch
    //   ->WaitForFetch
    //   x Conclude
    // WaitForFetch
    //   x Conclude

    public FetchTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if (Request.Parameters.ContainsKey("UID") || Request.Parameters.ContainsKey("Sequence-Set"))
        return ProcessFetch;
      else
        return ProcessParameterNotSetted;
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "parameter 'UID' or 'Sequence-Set' must be setted"));
    }

    private void ProcessFetch()
    {
      // TODO: sectionを指定したBODY[<section>], BODY.PEEK[<section>]への対応(レスポンスパーサが対応していない)
      if (Request.Parameters.ContainsKey("UID")) {
        // UID FETCH
        lastTag = Send("uid fetch",
                       Request.Parameters["UID"],
                       Request.Parameters["Message-Data-Items"]);
      }
      else if (Request.Parameters.ContainsKey("Sequence-Set")) {
        // FETCH
        lastTag = Send("fetch",
                       Request.Parameters["Sequence-Set"],
                       Request.Parameters["Message-Data-Items"]);
      }

      Transit(WaitForFetch);
    }

    private void WaitForFetch()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        ImapMessage message;
        if (ImapDataResponseConverter.FromFetchResponse(response as ImapDataResponse, out message)) {
          Trace.Dump(message, 0);
          messages.Add(message);
        }
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapMessageResponse(ImapTransactionStatus.Ok, messages.ToArray(), string.Format("fetched {0} message(s)", messages.Count)));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
    private List<ImapMessage> messages = new List<ImapMessage>();
  }
}
