using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class IdleDoneTransaction : TransactionBase {

    // ProcessNotIdling
    //   x Conclude
    // ProcessIdleIncapable
    //   x Conclude
    // ProcessIdleDone
    //   x Conclude

    public IdleDoneTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if ((Connection.ServerInfo.Capability.KnownCapabilities & ImapKnownCapabilities.Idle) != 0) {
        if (Connection.IsIdling)
          return ProcessIdleDone;
        else
          return ProcessNotIdling;
      }
      else {
        return ProcessIdleIncapable;
      }
    }

    private void ProcessNotIdling()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, "not idling"));
    }

    private void ProcessIdleIncapable()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.Incapable, "IDLE incapable"));
    }

    private void ProcessIdleDone()
    {
      // IDLEコマンドの継続要求応答に答える
      SendContinue("DONE");

      Conclude(new ImapResponse(ImapTransactionStatus.Ok, "done"));
    }
  }
}
