using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class NamespaceTransaction : TransactionBase {

    // ProcessNamespaceIncapable
    //   x Conclude
    // ProcessNamespace
    //   ->WaitForNamespace
    // WaitForNamespace
    //   x Conclude

    public NamespaceTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if ((Connection.ServerInfo.Capability.KnownCapabilities & ImapKnownCapabilities.Namespace) != 0) {
        @namespace = Connection.ServerInfo.Namespace; // reset if failed

        return ProcessNamespace;
      }
      else {
        return ProcessNamespaceIncapable;
      }
    }

    private void ProcessNamespaceIncapable()
    {
      Conclude(new ImapResponse(ImapTransactionStatus.Incapable, "NAMESPACE incapable"));
    }

    private void ProcessNamespace()
    {
      lastTag = Send("namespace");

      Transit(WaitForNamespace);
    }

    private void WaitForNamespace()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        ImapNamespace ns;
        if (ImapDataResponseConverter.FromNamespaceResponse(response as ImapDataResponse, out ns))
          @namespace = ns;
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Connection.ServerInfo.Namespace = @namespace;

          Conclude(new ImapNamespaceResponse(ImapTransactionStatus.Ok, @namespace, "comprehended namespace"));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
    private ImapNamespace @namespace = new ImapNamespace();
  }
}
