using System;
using System.Collections.Generic;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal class SearchTransaction : TransactionBase {

    // ProcessSearch
    //   ->WaitForSearch
    // WaitForSearch
    //   x Conclude

    public SearchTransaction(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      return ProcessSearch;
    }

    private void ProcessSearch()
    {
      if (Request.Parameters.ContainsKey("Search-Query"))
        lastTag = Send("search", Request.Parameters["Search-Query"]);
      else
        lastTag = Send("search", "all");

      Transit(WaitForSearch);
    }

    private void WaitForSearch()
    {
      ImapProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is ImapDataResponse) {
        int[] msgs;

        if (ImapDataResponseConverter.FromSearchResponse(response as ImapDataResponse, out msgs))
          messages = msgs;
      }
      else if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;

        if (status.ResponseCode != null) {
          string[] supportedCharset;
          if (ImapStatusResponseConverter.FromSearchResponse(status, out supportedCharset))
            this.supportedCharset = supportedCharset;
        }

        if (status.Tag != lastTag)
          return;

        if (status.Status == ImapProtocolStatus.OK) {
          Conclude(new ImapMessageListResponse(ImapTransactionStatus.Ok, messages, string.Format("matched {0} message(s)", messages.Length)));
        }
        else if (status.Status == ImapProtocolStatus.No) {
          var description = "search failed";
          if (supportedCharset != null) {
            description += string.Format(", supported charset is ", string.Join(", ", supportedCharset));
          }
          Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, description));
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private string lastTag = string.Empty;
    private int[] messages = new int[] {};
    private string[] supportedCharset = null;
  }
}
