using System;
using System.Collections.Generic;

using Smdn.Protocols.Pop3.Session;

namespace Smdn.Protocols.Pop3 {
  /// <summary>メールボックスの状態(メッセージ数、取得したメッセージ)を保持する</summary>
  /// <remarks>
  /// ImapMailboxのインスタンスはImapSessionで保持する
  /// このクラスの状態はSession層で管理し、それ以下の層では一切使用・参照しない
  /// </remarks>
  public class PopMailbox {
    internal Dictionary<int, PopMessage> ManagedMessages {
      get { return managedMessages; }
    }

    public PopMessage[] Messages {
      get { return (new List<PopMessage>(managedMessages.Values)).ToArray(); }
    }

    public int ExistsMessage {
      get { return existsMessage; }
      internal set { existsMessage = value; }
    }

    public int TotalOctets {
      get { return totalOctets; }
      internal set { totalOctets = value; }
    }

    internal bool Deleted {
      get { return deleted; }
      set { deleted = value; }
    }

    internal PopMailbox()
    {
    }

    public override string ToString()
    {
      return string.Format("ExistsMessage={0}, TotalOctets={1}", existsMessage, totalOctets);
    }

    private readonly Dictionary<int, PopMessage> managedMessages = new Dictionary<int, PopMessage>();
    private int existsMessage = 0;
    private int totalOctets = 0;
    private bool deleted = false;
  }
}
