using System;

namespace Smdn.Protocols.Pop3.Protocol {
  // Layers:
  //   Client.PopClient
  //   Client.PopClientAgent
  //   Session.PopSessionAgent
  //   Session.PopSession
  //   Transaction.PopTransactionAgent
  //   Transaction.PopTransaction/Transaction.PopRequest/Transaction.PopResponse
  // * Protocol.PopConnection/Protocol.PopProtocolCommand/Protocol.PopProtocolResponse
  //   (POP3)

  // ResponseTypes:
  //   PopProtocolResponse(abstract base)
  //     => handles all response
  //     PopStatusResponse
  //       => handles '+OK' or '-ERR' status indicator responses
  //     PopFollowingResponse
  //       => handles a line of following multi-line response after ImapStatusResponse
  //     PopTerminationResponse
  //       => handles a termination of multi-line response (CRLF.CRLF)
  //     PopContinuationRequest
  //       => handles 'RFC 1734 POP3 AUTHentication command' continuation request

  public abstract class PopProtocolResponse {
    public string Response {
      get { return response; }
    }

    protected PopProtocolResponse(string response)
    {
      if (response == null)
        throw new ArgumentNullException("response");

      this.response = response;
    }

    private string response = string.Empty;
  }
}