using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Sockets;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Session {
  // Layers:
  //   Client.PopClient
  //   Client.PopClientAgent
  //   Session.PopSessionAgent
  // * Session.PopSession
  //   Transaction.PopTransactionAgent
  //   Transaction.PopTransaction/Transaction.PopRequest/Transaction.PopResponse
  //   Protocol.PopConnection/Protocol.PopProtocolCommand/Protocol.PopProtocolResponse
  //   (POP3)

  public class PopSession {
    internal PopConnection Connection {
      get { return connection; }
    }

    internal int Id {
      get { return id; }
    }

    public PopServerInfo ServerInfo {
      get { return connection.ServerInfo; }
    }

    public PopUri HostUri {
      get { return connection.ServerInfo.Uri; }
    }

    public PopSessionState State {
      get { return state; }
      internal set { state = value; }
    }

    public int Timeout {
      get { return timeout; }
      set { timeout = value; }
    }

    private PopSession(PopConnection connection)
    {
      this.connection = connection;

      id = connection.Id;
      state = PopSessionState.NotConnected;
    }

    ~PopSession()
    {
    }

    internal void Close()
    {
      if (connection == null)
        return;

      Trace.Info("CID:{0} session closing", id);

      connection.Disconnect();

      state = PopSessionState.NotConnected;

      connection = null;

      Trace.Info("CID:{0} session closed", id);
    }

    static internal PopSession Start(PopUri hostUri, PopCreateAuthenticatedStreamCallback createAuthStreamCallback)
    {
      try {
        hostUri = new PopUri(hostUri.GetLeftPart(UriPartial.Authority));

        var connection = PopConnection.Connect(hostUri, createAuthStreamCallback);

        var session = new PopSession(connection);

        Trace.Info("CID:{0} session started", session.Id);

        return session;
      }
      catch (Exception ex) {
        Trace.Log(ex);

        return null;
      }
    }

    public override string ToString()
    {
      return string.Format("{{State={0}, Connection={1}}}",
                           state,
                           connection);
    }

    private PopConnection connection;
    private readonly int id = 0;
    private PopSessionState state;
    private int timeout = 32 * 1000;
  }
}
