using System;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class CompositeTransaction : TransactionBase {
    internal class InnerTransaction {
      public PopTransaction Transaction {
        get { return transaction; }
      }

      public bool ConcludeIfError {
        get { return concludeIfError; }
      }

      public InnerTransaction(PopTransaction transaction)
        : this(transaction, true)
      {
      }

      public InnerTransaction(PopTransaction transaction, bool concludeIfError)
      {
        this.transaction = transaction;
        this.concludeIfError = concludeIfError;
      }

      private readonly PopTransaction transaction;
      private readonly bool concludeIfError;
    }

    public CompositeTransaction(PopConnection connection, PopRequest request, InnerTransaction[] subTransactions)
      : base(connection, request)
    {
      this.subTransactions = subTransactions;
    }

    public override void Abort ()
    {
      currentTransaction.Transaction.Abort();

      base.Abort();
    }

    protected override ProcessTransactionDelegate Reset()
    {
      currentTransactionIndex = 0;
      currentTransaction = null;

      return ProcessSubTransaction;
    }

    private void ProcessSubTransaction()
    {
      if (Aborted)
        return;

      if (currentTransaction == null) {
        currentTransaction = subTransactions[currentTransactionIndex];
        currentTransaction.Transaction.ResetInternal();
      }

      currentTransaction.Transaction.Process();

      if (currentTransaction.Transaction.Response == null)
        return;

      if (!currentTransaction.ConcludeIfError ||
        ((int)currentTransaction.Transaction.Response.Status < 300)) {
        currentTransactionIndex++;
        if (currentTransactionIndex == subTransactions.Length) {
          Conclude(currentTransaction.Transaction.Response);
        }
        else {
          Trace.Log(currentTransaction.Transaction.Response, currentTransaction.Transaction);

          currentTransaction = null;
        }
      }
      else {
        Conclude(currentTransaction.Transaction.Response);
      }
    }

    private int currentTransactionIndex;
    private InnerTransaction currentTransaction;
    private InnerTransaction[] subTransactions;
  }
}
