using System;
using System.Collections.Generic;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class ListTransaction : TransactionBase {

    // ProcessList
    //   ->WaitForList
    // WaitForList
    //   x Conclude
    // RetrieveList
    //   x Conclude

    public ListTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      listMultiple = !Request.Parameters.ContainsKey("Message-Number");

      return ProcessList;
    }

    private void ProcessList()
    {
      if (listMultiple)
        Send("LIST");
      else
        Send("LIST", Request.Parameters["Message-Number"]);

      Transit(WaitForList);
    }

    private void WaitForList()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          if (listMultiple) {
            Transit(RetrieveList);
          }
          else {
            PopScanList scanList;
            if (RetrieveScanList(status, out scanList))
              Conclude(new PopScanListResponse(PopTransactionStatus.Ok, new[] {scanList}, "retrieved a scan list"));
            else
              Conclude(new PopResponse(PopTransactionStatus.BadRequest, "invalid scan list format"));
          }
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private void RetrieveList()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopFollowingResponse) {
        PopScanList scanList;
        if (RetrieveScanList(response as PopFollowingResponse, out scanList))
          scanLists.Add(scanList);
      }
      else if (response is PopTerminationResponse) {
        Conclude(new PopScanListResponse(PopTransactionStatus.Ok,
                                         scanLists.ToArray(),
                                         "retrieved scan lists"));
      }
    }

    private bool RetrieveScanList(PopStatusResponse status, out PopScanList scanList)
    {
      return RetrieveScanList(status.Message, out scanList);
    }

    private bool RetrieveScanList(PopFollowingResponse following, out PopScanList scanList)
    {
      return RetrieveScanList(following.Text, out scanList);
    }

    private bool RetrieveScanList(string text, out PopScanList scanList)
    {
      scanList = null;

      var resp = text.Split(new[] {' '}, StringSplitOptions.None);

      if (resp.Length < 2)
        return false;

      int messageNumber;

      if (!int.TryParse(resp[0], out messageNumber))
        return false;

      int octets;

      if (!int.TryParse(resp[1], out octets))
        return false;

      scanList = new PopScanList(messageNumber, octets);

      return true;
    }

    private bool listMultiple;
    private readonly List<PopScanList> scanLists = new List<PopScanList>();
  }
}
